/* Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
'use strict';

(function() {
  angular
    .module('ngFormFixes', [])
    .directive('ngForm', ['$parse', '$timeout', function($parse, $timeout) {
      return {
        link: linkFunction
      };

      function linkFunction($scope, $element, $attrs) {

        var submitButton = findSubmitButton();

        $scope.$on('$destroy', function() {
          $element.off('keydown', handleKeydown);
          angular.element(submitButton).off('click', handleClick);
        });

        function handleKeydown(e) {
          var keyCode = e.keyCode || e.which;
          if (keyCode === 13) {
            var nodeType = e.target.nodeName.toLowerCase();
            if (nodeType === 'input') {
              if ($attrs.ngSubmit) {
                $parse($attrs.ngSubmit)($scope, {
                  $event: e
                });
                e.stopPropagation();
                e.preventDefault();
              } else if (submitButton) {
                submitButton.click();
                e.stopPropagation();
                e.preventDefault();
              }
            }
          }
        }

        function handleClick(e) {
          if ($attrs.ngSubmit && angular.element(this).attr('ng-click') === undefined) {
            $parse($attrs.ngSubmit)($scope, {
              $event: e
            });
            e.stopPropagation();
            e.preventDefault();

            $timeout(function() {
              if ($scope[$attrs.ngForm || $attrs.name]) {
                $scope[$attrs.ngForm || $attrs.name].$submitted = true;
              }
              $element.addClass('ng-submitted');
            });
          }
        }

        // bind Enter key
        $element.on('keydown', handleKeydown);

        // bind Submit click to run itself or ngSubmit
        angular.element(submitButton).on('click', handleClick);

        // internal
        function findSubmitButton() {
          var $buttons = [$element.find('button'), $element.find('input')];

          for (var i = 0; i < $buttons.length; i++) {
            for (var n = 0; n < $buttons[i].length; n++) {
              var $current = $buttons[i][n];
              if ($current.getAttribute('type') && $current.getAttribute('type').toLowerCase() === 'submit') {
                return $current;
              }

            }
          }
        }
      }

    }])

  .directive('onEnter', ['$parse', function($parse) {
    return {
      link: function($scope, $element, $attrs) {

        function handleKeyup(e) {
          var keyCode = e.keyCode || e.which;

          if (keyCode === 13 && $attrs.onEnter) {
            $parse($attrs.onEnter)($scope, {
              $event: e
            });
          }
        }

        $element.on('keyup', handleKeyup);

        $scope.$on('$destroy', function() {
          $element.off('keyup', handleKeyup);
        });

      }
    };
  }]);
})();